/*
 * deinja
 * Copyright (C) 2018 wtetsu
 * https://github.com/wtetsu/deinja
 *
 * Originally based on deinflector
 * Copyright (C) 2016 James Kirk
 * https://github.com/Jimeux/deinflector
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const Form = require("./form");

class Inflection {
  constructor(inflection, base, form) {
    this.inflection = inflection;
    this.base = base;
    this.form = form;
  }
}

const SURU_INFLECTIONS = [
  new Inflection("しませんでした", "する", Form.POLITE_PAST_NEGATIVE),
  new Inflection("しましょう", "する", Form.POLITE_VOLITIONAL),
  new Inflection("した", "する", Form.PAST),
  new Inflection("して", "する", Form.TE),
  new Inflection("しろ", "する", Form.IMPERATIVE),
  new Inflection("せず", "する", Form.ZU),
  new Inflection("せよ", "する", Form.IMPERATIVE),
  new Inflection("しすぎる", "する", Form.SUGIRU),
  new Inflection("しちゃう", "する", Form.CHAU),
  new Inflection("しなさい", "する", Form.NASAI),
  new Inflection("しました", "する", Form.POLITE_PAST),
  new Inflection("しません", "する", Form.POLITE_NEGATIVE),
  new Inflection("させる", "する", Form.CAUSATIVE),
  new Inflection("される", "する", Form.PASSIVE),
  new Inflection("しそう", "する", Form.SOU),
  new Inflection("したい", "する", Form.TAI),
  new Inflection("したら", "する", Form.TARA),
  new Inflection("したり", "する", Form.TARI),
  new Inflection("しない", "する", Form.NEGATIVE),
  new Inflection("します", "する", Form.POLITE),
  new Inflection("しよう", "する", Form.VOLITIONAL)
];

const KURU_INFLECTIONS = [
  new Inflection("来すぎる", "来る", Form.SUGIRU),
  new Inflection("来ませんでした", "来る", Form.POLITE_PAST_NEGATIVE),
  new Inflection("来ましょう", "来る", Form.POLITE_VOLITIONAL),
  new Inflection("来ちゃう", "来る", Form.CHAU),
  new Inflection("来なさい", "来る", Form.NASAI),
  new Inflection("来ました", "来る", Form.POLITE_PAST),
  new Inflection("来ません", "来る", Form.POLITE_NEGATIVE),
  new Inflection("来させる", "来る", Form.CAUSATIVE),
  new Inflection("来られる", "来る", Form.POTENTIAL_OR_PASSIVE),
  new Inflection("来そう", "来る", Form.SOU),
  new Inflection("来たい", "来る", Form.TAI),
  new Inflection("来たら", "来る", Form.TARA),
  new Inflection("来たり", "来る", Form.TARI),
  new Inflection("来ます", "来る", Form.POLITE),
  new Inflection("来ない", "来る", Form.NEGATIVE),
  new Inflection("来よう", "来る", Form.VOLITIONAL),
  new Inflection("来れる", "来る", Form.POTENTIAL),
  new Inflection("来た", "来る", Form.PAST),
  new Inflection("来て", "来る", Form.TE),
  new Inflection("来い", "来る", Form.IMPERATIVE),
  new Inflection("来ず", "来る", Form.ZU),
  new Inflection("きすぎる", "くる", Form.SUGIRU),
  new Inflection("きませんでした", "くる", Form.POLITE_PAST_NEGATIVE),
  new Inflection("きましょう", "くる", Form.POLITE_VOLITIONAL),
  new Inflection("きちゃう", "くる", Form.CHAU),
  new Inflection("きなさい", "くる", Form.NASAI),
  new Inflection("きました", "くる", Form.POLITE_PAST),
  new Inflection("きません", "くる", Form.POLITE_NEGATIVE),
  new Inflection("こさせる", "くる", Form.CAUSATIVE),
  new Inflection("こられる", "くる", Form.POTENTIAL_OR_PASSIVE),
  new Inflection("きそう", "くる", Form.SOU),
  new Inflection("きたい", "くる", Form.TAI),
  new Inflection("きたら", "くる", Form.TARA),
  new Inflection("きたり", "くる", Form.TARI),
  new Inflection("きます", "くる", Form.POLITE),
  new Inflection("こない", "くる", Form.NEGATIVE),
  new Inflection("こよう", "くる", Form.VOLITIONAL),
  new Inflection("これる", "くる", Form.POTENTIAL),
  new Inflection("きた", "くる", Form.PAST),
  new Inflection("きて", "くる", Form.TE),
  new Inflection("こい", "くる", Form.IMPERATIVE),
  new Inflection("こず", "くる", Form.ZU)
];

const IKU_INFLECTIONS = [
  new Inflection("いって", "いく", Form.TE),
  new Inflection("ゆって", "ゆく", Form.TE),
  new Inflection("行って", "行く", Form.TE),
  new Inflection("征って", "征く", Form.TE),
  new Inflection("逝って", "逝く", Form.TE),
  new Inflection("往って", "往く", Form.TE),
  new Inflection("いった", "いく", Form.PAST),
  new Inflection("ゆった", "ゆく", Form.PAST),
  new Inflection("行った", "行く", Form.PAST),
  new Inflection("征った", "征く", Form.PAST),
  new Inflection("逝った", "逝く", Form.PAST),
  new Inflection("往った", "往く", Form.PAST),
  new Inflection("いっちゃう", "いく", Form.CHAU),
  new Inflection("ゆっちゃう", "ゆく", Form.CHAU),
  new Inflection("行っちゃう", "行く", Form.CHAU),
  new Inflection("征っちゃう", "征く", Form.CHAU),
  new Inflection("逝っちゃう", "逝く", Form.CHAU),
  new Inflection("往っちゃう", "往く", Form.CHAU),
  new Inflection("いったら", "いく", Form.TARA),
  new Inflection("ゆったら", "ゆく", Form.TARA),
  new Inflection("行ったら", "行く", Form.TARA),
  new Inflection("征ったら", "征く", Form.TARA),
  new Inflection("逝ったら", "逝く", Form.TARA),
  new Inflection("往ったら", "往く", Form.TARA),
  new Inflection("いったり", "いく", Form.TARI),
  new Inflection("ゆったり", "ゆく", Form.TARI),
  new Inflection("行ったり", "行く", Form.TARI),
  new Inflection("征ったり", "征く", Form.TARI),
  new Inflection("逝ったり", "逝く", Form.TARI),
  new Inflection("往ったり", "往く", Form.TARI),
  new Inflection("いきませんでした", "いく", Form.POLITE_PAST_NEGATIVE),
  new Inflection("ゆきませんでした", "ゆく", Form.POLITE_PAST_NEGATIVE),
  new Inflection("行きませんでした", "行く", Form.POLITE_PAST_NEGATIVE),
  new Inflection("征きませんでした", "征く", Form.POLITE_PAST_NEGATIVE),
  new Inflection("逝きませんでした", "逝く", Form.POLITE_PAST_NEGATIVE),
  new Inflection("往きませんでした", "往く", Form.POLITE_PAST_NEGATIVE),
  new Inflection("いきましょう", "いく", Form.POLITE_VOLITIONAL),
  new Inflection("ゆきましょう", "ゆく", Form.POLITE_VOLITIONAL),
  new Inflection("行きましょう", "行く", Form.POLITE_VOLITIONAL),
  new Inflection("征きましょう", "征く", Form.POLITE_VOLITIONAL),
  new Inflection("逝きましょう", "逝く", Form.POLITE_VOLITIONAL),
  new Inflection("往きましょう", "往く", Form.POLITE_VOLITIONAL),
  new Inflection("いけ", "いく", Form.IMPERATIVE),
  new Inflection("ゆけ", "ゆく", Form.IMPERATIVE),
  new Inflection("行け", "行く", Form.IMPERATIVE),
  new Inflection("征け", "征く", Form.IMPERATIVE),
  new Inflection("逝け", "逝く", Form.IMPERATIVE),
  new Inflection("往け", "往く", Form.IMPERATIVE),
  new Inflection("いかず", "いく", Form.ZU),
  new Inflection("ゆかず", "ゆく", Form.ZU),
  new Inflection("行かず", "行く", Form.ZU),
  new Inflection("征かず", "征く", Form.ZU),
  new Inflection("逝かず", "逝く", Form.ZU),
  new Inflection("往かず", "往く", Form.ZU),
  new Inflection("いきすぎる", "いく", Form.SUGIRU),
  new Inflection("ゆきすぎる", "ゆく", Form.SUGIRU),
  new Inflection("行きすぎる", "行く", Form.SUGIRU),
  new Inflection("征きすぎる", "征く", Form.SUGIRU),
  new Inflection("逝きすぎる", "逝く", Form.SUGIRU),
  new Inflection("往きすぎる", "往く", Form.SUGIRU),
  new Inflection("いきなさい", "いく", Form.NASAI),
  new Inflection("ゆきなさい", "ゆく", Form.NASAI),
  new Inflection("行きなさい", "行く", Form.NASAI),
  new Inflection("征きなさい", "征く", Form.NASAI),
  new Inflection("逝きなさい", "逝く", Form.NASAI),
  new Inflection("往きなさい", "往く", Form.NASAI),
  new Inflection("いきました", "いく", Form.POLITE_PAST),
  new Inflection("ゆきました", "ゆく", Form.POLITE_PAST),
  new Inflection("行きました", "行く", Form.POLITE_PAST),
  new Inflection("征きました", "征く", Form.POLITE_PAST),
  new Inflection("逝きました", "逝く", Form.POLITE_PAST),
  new Inflection("往きました", "往く", Form.POLITE_PAST),
  new Inflection("いきません", "いく", Form.POLITE_NEGATIVE),
  new Inflection("ゆきません", "ゆく", Form.POLITE_NEGATIVE),
  new Inflection("行きません", "行く", Form.POLITE_NEGATIVE),
  new Inflection("征きません", "征く", Form.POLITE_NEGATIVE),
  new Inflection("逝きません", "逝く", Form.POLITE_NEGATIVE),
  new Inflection("往きません", "往く", Form.POLITE_NEGATIVE),
  new Inflection("いかせる", "いく", Form.CAUSATIVE),
  new Inflection("ゆかせる", "ゆく", Form.CAUSATIVE),
  new Inflection("行かせる", "行く", Form.CAUSATIVE),
  new Inflection("征かせる", "征く", Form.CAUSATIVE),
  new Inflection("逝かせる", "逝く", Form.CAUSATIVE),
  new Inflection("往かせる", "往く", Form.CAUSATIVE),
  new Inflection("いかれる", "いく", Form.PASSIVE),
  new Inflection("ゆかれる", "ゆく", Form.PASSIVE),
  new Inflection("行かれる", "行く", Form.PASSIVE),
  new Inflection("征かれる", "征く", Form.PASSIVE),
  new Inflection("逝かれる", "逝く", Form.PASSIVE),
  new Inflection("往かれる", "往く", Form.PASSIVE),
  new Inflection("いきそう", "いく", Form.SOU),
  new Inflection("ゆきそう", "ゆく", Form.SOU),
  new Inflection("行きそう", "行く", Form.SOU),
  new Inflection("征きそう", "征く", Form.SOU),
  new Inflection("逝きそう", "逝く", Form.SOU),
  new Inflection("往きそう", "往く", Form.SOU),
  new Inflection("いきたい", "いく", Form.TAI),
  new Inflection("ゆきたい", "ゆく", Form.TAI),
  new Inflection("行きたい", "行く", Form.TAI),
  new Inflection("征きたい", "征く", Form.TAI),
  new Inflection("逝きたい", "逝く", Form.TAI),
  new Inflection("往きたい", "往く", Form.TAI),
  new Inflection("いかない", "いく", Form.NEGATIVE),
  new Inflection("ゆかない", "ゆく", Form.NEGATIVE),
  new Inflection("行かない", "行く", Form.NEGATIVE),
  new Inflection("征かない", "征く", Form.NEGATIVE),
  new Inflection("逝かない", "逝く", Form.NEGATIVE),
  new Inflection("往かない", "往く", Form.NEGATIVE),
  new Inflection("いきます", "いく", Form.POLITE),
  new Inflection("ゆきます", "ゆく", Form.POLITE),
  new Inflection("行きます", "行く", Form.POLITE),
  new Inflection("征きます", "征く", Form.POLITE),
  new Inflection("逝きます", "逝く", Form.POLITE),
  new Inflection("往きます", "往く", Form.POLITE),
  new Inflection("いこう", "いく", Form.VOLITIONAL),
  new Inflection("ゆこう", "ゆく", Form.VOLITIONAL),
  new Inflection("行こう", "行く", Form.VOLITIONAL),
  new Inflection("征こう", "征く", Form.VOLITIONAL),
  new Inflection("逝こう", "逝く", Form.VOLITIONAL),
  new Inflection("往こう", "往く", Form.VOLITIONAL)
];

const SPECIAL_INFLECTIONS = [
  new Inflection("訪うて", "訪う", Form.TE),
  new Inflection("問うて", "問う", Form.TE),
  new Inflection("とうて", "とう", Form.TE),
  new Inflection("訪うた", "訪う", Form.PAST),
  new Inflection("問うた", "問う", Form.PAST),
  new Inflection("とうた", "とう", Form.PAST),
  new Inflection("乞うて", "乞う", Form.TE),
  new Inflection("恋うて", "恋う", Form.TE),
  new Inflection("請うて", "請う", Form.TE),
  new Inflection("こうて", "こう", Form.TE),
  new Inflection("乞うた", "乞う", Form.PAST),
  new Inflection("恋うた", "恋う", Form.PAST),
  new Inflection("請うた", "請う", Form.PAST),
  new Inflection("こうた", "こう", Form.PAST)
];

const ICHIDAN_INFLECTIONS = [
  new Inflection("ません", "る", Form.POLITE_NEGATIVE),
  new Inflection("ました", "る", Form.POLITE_PAST),
  new Inflection("らせる", "る", Form.CAUSATIVE),
  new Inflection("らない", "る", Form.NEGATIVE),
  new Inflection("られる", "る", Form.POTENTIAL_OR_PASSIVE),
  new Inflection("られ", "る", Form.POTENTIAL_OR_PASSIVE),
  new Inflection("たら", "る", Form.TARA),
  new Inflection("たり", "る", Form.TARI),
  new Inflection("そう", "る", Form.SOU),
  new Inflection("たい", "る", Form.TAI),
  new Inflection("ない", "る", Form.NEGATIVE),
  new Inflection("ます", "る", Form.POLITE),
  new Inflection("よう", "る", Form.VOLITIONAL),
  new Inflection("れば", "る", Form.BA),
  new Inflection("ず", "る", Form.ZU),
  new Inflection("て", "る", Form.TE),
  new Inflection("ろ", "る", Form.IMPERATIVE),
  new Inflection("た", "る", Form.PAST),
  new Inflection("ちゃう", "る", Form.CHAU),
  new Inflection("ませんでした", "る", Form.POLITE_PAST_NEGATIVE),
  new Inflection("ましょう", "る", Form.POLITE_VOLITIONAL),
  new Inflection("させる", "る", Form.CAUSATIVE),
  new Inflection("すぎる", "る", Form.SUGIRU),
  new Inflection("なさい", "る", Form.NASAI)
];

const ADJECTIVE_INFLECTIONS = [
  new Inflection("くありませんでした", "い", Form.POLITE_PAST_NEGATIVE),
  new Inflection("くありません", "い", Form.POLITE_NEGATIVE),
  new Inflection("かったら", "い", Form.TARA),
  new Inflection("かったり", "い", Form.TARI),
  new Inflection("かった", "い", Form.PAST),
  new Inflection("くない", "い", Form.NEGATIVE),
  new Inflection("すぎる", "い", Form.SUGIRU),
  new Inflection("ければ", "い", Form.BA),
  new Inflection("くて", "い", Form.TE),
  new Inflection("そう", "い", Form.SOU),
  new Inflection("く", "い", Form.ADV),
  new Inflection("さ", "い", Form.NOUN)
];

const GODAN_INFLECTIONS = [
  new Inflection("いませんでした", "う", Form.POLITE_PAST_NEGATIVE),
  new Inflection("きませんでした", "く", Form.POLITE_PAST_NEGATIVE),
  new Inflection("ぎませんでした", "ぐ", Form.POLITE_PAST_NEGATIVE),
  new Inflection("しませんでした", "す", Form.POLITE_PAST_NEGATIVE),
  new Inflection("ちませんでした", "つ", Form.POLITE_PAST_NEGATIVE),
  new Inflection("にませんでした", "ぬ", Form.POLITE_PAST_NEGATIVE),
  new Inflection("びませんでした", "ぶ", Form.POLITE_PAST_NEGATIVE),
  new Inflection("みませんでした", "む", Form.POLITE_PAST_NEGATIVE),
  new Inflection("りませんでした", "る", Form.POLITE_PAST_NEGATIVE),
  new Inflection("いましょう", "う", Form.POLITE_VOLITIONAL),
  new Inflection("きましょう", "く", Form.POLITE_VOLITIONAL),
  new Inflection("ぎましょう", "ぐ", Form.POLITE_VOLITIONAL),
  new Inflection("しましょう", "す", Form.POLITE_VOLITIONAL),
  new Inflection("ちましょう", "つ", Form.POLITE_VOLITIONAL),
  new Inflection("にましょう", "ぬ", Form.POLITE_VOLITIONAL),
  new Inflection("びましょう", "ぶ", Form.POLITE_VOLITIONAL),
  new Inflection("みましょう", "む", Form.POLITE_VOLITIONAL),
  new Inflection("りましょう", "る", Form.POLITE_VOLITIONAL),
  new Inflection("いじゃう", "ぐ", Form.CHAU),
  new Inflection("いすぎる", "う", Form.SUGIRU),
  new Inflection("いちゃう", "く", Form.CHAU),
  new Inflection("いなさい", "う", Form.NASAI),
  new Inflection("いました", "う", Form.POLITE_PAST),
  new Inflection("いません", "う", Form.POLITE_NEGATIVE),
  new Inflection("きすぎる", "く", Form.SUGIRU),
  new Inflection("ぎすぎる", "ぐ", Form.SUGIRU),
  new Inflection("きなさい", "く", Form.NASAI),
  new Inflection("ぎなさい", "ぐ", Form.NASAI),
  new Inflection("きました", "く", Form.POLITE_PAST),
  new Inflection("ぎました", "ぐ", Form.POLITE_PAST),
  new Inflection("きません", "く", Form.POLITE_NEGATIVE),
  new Inflection("ぎません", "ぐ", Form.POLITE_NEGATIVE),
  new Inflection("しすぎる", "す", Form.SUGIRU),
  new Inflection("しちゃう", "す", Form.CHAU),
  new Inflection("しなさい", "す", Form.NASAI),
  new Inflection("しました", "す", Form.POLITE_PAST),
  new Inflection("しません", "す", Form.POLITE_NEGATIVE),
  new Inflection("ちすぎる", "つ", Form.SUGIRU),
  new Inflection("ちなさい", "つ", Form.NASAI),
  new Inflection("ちました", "つ", Form.POLITE_PAST),
  new Inflection("ちません", "つ", Form.POLITE_NEGATIVE),
  new Inflection("っちゃう", "う", Form.CHAU),
  new Inflection("っちゃう", "つ", Form.CHAU),
  new Inflection("っちゃう", "る", Form.CHAU),
  new Inflection("にすぎる", "ぬ", Form.SUGIRU),
  new Inflection("になさい", "ぬ", Form.NASAI),
  new Inflection("にました", "ぬ", Form.POLITE_PAST),
  new Inflection("にません", "ぬ", Form.POLITE_NEGATIVE),
  new Inflection("びすぎる", "ぶ", Form.SUGIRU),
  new Inflection("びなさい", "ぶ", Form.NASAI),
  new Inflection("びました", "ぶ", Form.POLITE_PAST),
  new Inflection("びません", "ぶ", Form.POLITE_NEGATIVE),
  new Inflection("みすぎる", "む", Form.SUGIRU),
  new Inflection("みなさい", "む", Form.NASAI),
  new Inflection("みました", "む", Form.POLITE_PAST),
  new Inflection("みません", "む", Form.POLITE_NEGATIVE),
  new Inflection("りすぎる", "る", Form.SUGIRU),
  new Inflection("りなさい", "る", Form.NASAI),
  new Inflection("りました", "る", Form.POLITE_PAST),
  new Inflection("りません", "る", Form.POLITE_NEGATIVE),
  new Inflection("んじゃう", "ぬ", Form.CHAU),
  new Inflection("んじゃう", "ぶ", Form.CHAU),
  new Inflection("んじゃう", "む", Form.CHAU),
  new Inflection("いそう", "う", Form.SOU),
  new Inflection("いたい", "う", Form.TAI),
  new Inflection("いたら", "く", Form.TARA),
  new Inflection("いだら", "ぐ", Form.TARA),
  new Inflection("いたり", "く", Form.TARI),
  new Inflection("いだり", "ぐ", Form.TARI),
  new Inflection("います", "う", Form.POLITE),
  new Inflection("かせる", "く", Form.CAUSATIVE),
  new Inflection("がせる", "ぐ", Form.CAUSATIVE),
  new Inflection("かない", "く", Form.NEGATIVE),
  new Inflection("がない", "ぐ", Form.NEGATIVE),
  new Inflection("かれる", "く", Form.PASSIVE),
  new Inflection("がれる", "ぐ", Form.PASSIVE),
  new Inflection("きそう", "く", Form.SOU),
  new Inflection("ぎそう", "ぐ", Form.SOU),
  new Inflection("きたい", "く", Form.TAI),
  new Inflection("ぎたい", "ぐ", Form.TAI),
  new Inflection("きます", "く", Form.POLITE),
  new Inflection("ぎます", "ぐ", Form.POLITE),
  new Inflection("させる", "す", Form.CAUSATIVE),
  new Inflection("さない", "す", Form.NEGATIVE),
  new Inflection("される", "す", Form.PASSIVE_OR_CAUSATIVE),
  new Inflection("しそう", "す", Form.SOU),
  new Inflection("したい", "す", Form.TAI),
  new Inflection("したら", "す", Form.TARA),
  new Inflection("したり", "す", Form.TARI),
  new Inflection("します", "す", Form.POLITE),
  new Inflection("たせる", "つ", Form.CAUSATIVE),
  new Inflection("たない", "つ", Form.NEGATIVE),
  new Inflection("たれる", "つ", Form.PASSIVE),
  new Inflection("ちそう", "つ", Form.SOU),
  new Inflection("ちたい", "つ", Form.TAI),
  new Inflection("ちます", "つ", Form.POLITE),
  new Inflection("ったら", "う", Form.TARA),
  new Inflection("ったら", "つ", Form.TARA),
  new Inflection("ったら", "る", Form.TARA),
  new Inflection("ったり", "う", Form.TARI),
  new Inflection("ったり", "つ", Form.TARI),
  new Inflection("ったり", "る", Form.TARI),
  new Inflection("なせる", "ぬ", Form.CAUSATIVE),
  new Inflection("なない", "ぬ", Form.NEGATIVE),
  new Inflection("なれる", "ぬ", Form.PASSIVE),
  new Inflection("にそう", "ぬ", Form.SOU),
  new Inflection("にたい", "ぬ", Form.TAI),
  new Inflection("にます", "ぬ", Form.POLITE),
  new Inflection("ばせる", "ぶ", Form.CAUSATIVE),
  new Inflection("ばない", "ぶ", Form.NEGATIVE),
  new Inflection("ばれる", "ぶ", Form.PASSIVE),
  new Inflection("びそう", "ぶ", Form.SOU),
  new Inflection("びたい", "ぶ", Form.TAI),
  new Inflection("びます", "ぶ", Form.POLITE),
  new Inflection("ませる", "む", Form.CAUSATIVE),
  new Inflection("まない", "む", Form.NEGATIVE),
  new Inflection("まれる", "む", Form.PASSIVE),
  new Inflection("みそう", "む", Form.SOU),
  new Inflection("みたい", "む", Form.TAI),
  new Inflection("みます", "む", Form.POLITE),
  new Inflection("りそう", "る", Form.SOU),
  new Inflection("りたい", "る", Form.TAI),
  new Inflection("ります", "る", Form.POLITE),
  new Inflection("わせる", "う", Form.CAUSATIVE),
  new Inflection("わない", "う", Form.NEGATIVE),
  new Inflection("われる", "う", Form.PASSIVE),
  new Inflection("んだら", "ぬ", Form.TARA),
  new Inflection("んだら", "ぶ", Form.TARA),
  new Inflection("んだら", "む", Form.TARA),
  new Inflection("んだり", "ぬ", Form.TARI),
  new Inflection("んだり", "ぶ", Form.TARI),
  new Inflection("んだり", "む", Form.TARI),
  new Inflection("いた", "く", Form.PAST),
  new Inflection("いだ", "ぐ", Form.PAST),
  new Inflection("いて", "く", Form.TE),
  new Inflection("いで", "ぐ", Form.TE),
  new Inflection("えば", "う", Form.BA),
  new Inflection("える", "う", Form.POTENTIAL),
  new Inflection("おう", "う", Form.VOLITIONAL),
  new Inflection("かず", "く", Form.ZU),
  new Inflection("がず", "ぐ", Form.ZU),
  new Inflection("けば", "く", Form.BA),
  new Inflection("げば", "ぐ", Form.BA),
  new Inflection("ける", "く", Form.POTENTIAL),
  new Inflection("げる", "ぐ", Form.POTENTIAL),
  new Inflection("こう", "く", Form.VOLITIONAL),
  new Inflection("ごう", "ぐ", Form.VOLITIONAL),
  new Inflection("さず", "す", Form.ZU),
  new Inflection("した", "す", Form.PAST),
  new Inflection("して", "す", Form.TE),
  new Inflection("せば", "す", Form.BA),
  new Inflection("せる", "す", Form.POTENTIAL),
  new Inflection("そう", "す", Form.VOLITIONAL),
  new Inflection("たず", "つ", Form.ZU),
  new Inflection("った", "う", Form.PAST),
  new Inflection("った", "つ", Form.PAST),
  new Inflection("った", "る", Form.PAST),
  new Inflection("って", "う", Form.TE),
  new Inflection("って", "つ", Form.TE),
  new Inflection("って", "る", Form.TE),
  new Inflection("てば", "つ", Form.BA),
  new Inflection("てる", "つ", Form.POTENTIAL),
  new Inflection("とう", "つ", Form.VOLITIONAL),
  new Inflection("なず", "ぬ", Form.ZU),
  new Inflection("ねば", "ぬ", Form.BA),
  new Inflection("ねる", "ぬ", Form.POTENTIAL),
  new Inflection("のう", "ぬ", Form.VOLITIONAL),
  new Inflection("ばず", "ぶ", Form.ZU),
  new Inflection("べば", "ぶ", Form.BA),
  new Inflection("べる", "ぶ", Form.POTENTIAL),
  new Inflection("ぼう", "ぶ", Form.VOLITIONAL),
  new Inflection("まず", "む", Form.ZU),
  new Inflection("めば", "む", Form.BA),
  new Inflection("める", "む", Form.POTENTIAL),
  new Inflection("もう", "む", Form.VOLITIONAL),
  new Inflection("らず", "る", Form.ZU),
  new Inflection("れば", "る", Form.BA),
  new Inflection("れる", "る", Form.POTENTIAL),
  new Inflection("ろう", "る", Form.VOLITIONAL),
  new Inflection("わず", "う", Form.ZU),
  new Inflection("んだ", "ぬ", Form.PAST),
  new Inflection("んだ", "ぶ", Form.PAST),
  new Inflection("んだ", "む", Form.PAST),
  new Inflection("んで", "ぬ", Form.TE),
  new Inflection("んで", "ぶ", Form.TE),
  new Inflection("んで", "む", Form.TE),
  new Inflection("い", "いる", Form.MASU_STEM),
  new Inflection("い", "う", Form.MASU_STEM),
  new Inflection("え", "う", Form.IMPERATIVE),
  new Inflection("え", "える", Form.MASU_STEM),
  new Inflection("き", "きる", Form.MASU_STEM),
  new Inflection("き", "く", Form.MASU_STEM),
  new Inflection("ぎ", "ぎる", Form.MASU_STEM),
  new Inflection("ぎ", "ぐ", Form.MASU_STEM),
  new Inflection("け", "く", Form.IMPERATIVE),
  new Inflection("け", "ける", Form.MASU_STEM),
  new Inflection("げ", "ぐ", Form.IMPERATIVE),
  new Inflection("げ", "げる", Form.MASU_STEM),
  new Inflection("し", "す", Form.MASU_STEM),
  new Inflection("じ", "じる", Form.MASU_STEM),
  new Inflection("せ", "す", Form.IMPERATIVE),
  new Inflection("せ", "せる", Form.MASU_STEM),
  new Inflection("ぜ", "ぜる", Form.MASU_STEM),
  new Inflection("ち", "ちる", Form.MASU_STEM),
  new Inflection("ち", "つ", Form.MASU_STEM),
  new Inflection("て", "つ", Form.IMPERATIVE),
  new Inflection("て", "てる", Form.MASU_STEM),
  new Inflection("で", "でる", Form.MASU_STEM),
  new Inflection("に", "にる", Form.MASU_STEM),
  new Inflection("に", "ぬ", Form.MASU_STEM),
  new Inflection("ね", "ぬ", Form.IMPERATIVE),
  new Inflection("ね", "ねる", Form.MASU_STEM),
  new Inflection("ひ", "ひる", Form.MASU_STEM),
  new Inflection("び", "びる", Form.MASU_STEM),
  new Inflection("び", "ぶ", Form.MASU_STEM),
  new Inflection("へ", "へる", Form.MASU_STEM),
  new Inflection("べ", "ぶ", Form.IMPERATIVE),
  new Inflection("べ", "べる", Form.MASU_STEM),
  new Inflection("み", "みる", Form.MASU_STEM),
  new Inflection("み", "む", Form.MASU_STEM),
  new Inflection("め", "む", Form.IMPERATIVE),
  new Inflection("め", "める", Form.MASU_STEM),
  new Inflection("り", "りる", Form.MASU_STEM),
  new Inflection("り", "る", Form.MASU_STEM),
  new Inflection("れ", "る", Form.IMPERATIVE),
  new Inflection("れ", "れる", Form.MASU_STEM),
  new Inflection("れ", "れる", Form.MASU_STEM),
  new Inflection("らせる", "る", Form.CAUSATIVE),
  new Inflection("らない", "る", Form.NEGATIVE),
  new Inflection("られる", "る", Form.POTENTIAL_OR_PASSIVE),
  new Inflection("られ", "る", Form.POTENTIAL_OR_PASSIVE),
  new Inflection("れば", "る", Form.BA)
];

const ICHIDAN = {
  い: true,
  き: true,
  ぎ: true,
  し: true,
  じ: true,
  ち: true,
  ぢ: true,
  に: true,
  ひ: true,
  び: true,
  ぴ: true,
  み: true,
  り: true,
  イ: true,
  キ: true,
  ギ: true,
  シ: true,
  ジ: true,
  チ: true,
  ヂ: true,
  ニ: true,
  ヒ: true,
  ビ: true,
  ピ: true,
  ミ: true,
  リ: true,
  え: true,
  け: true,
  げ: true,
  せ: true,
  ぜ: true,
  て: true,
  で: true,
  ね: true,
  へ: true,
  べ: true,
  ぺ: true,
  め: true,
  れ: true,
  エ: true,
  ケ: true,
  ゲ: true,
  セ: true,
  ゼ: true,
  テ: true,
  デ: true,
  ネ: true,
  ヘ: true,
  ベ: true,
  ペ: true,
  メ: true,
  レ: true
};

const BOGUS_INFLECTIONS = [
  "あつ",
  "あぐ",
  "あす",
  "あず",
  "あぬ",
  "あふ",
  "あぶ",
  "いつ",
  "えつ",
  "きう",
  "くなかう",
  "くなかる",
  "させられる",
  "しつ",
  "しう",
  "じう",
  "じつ",
  "たいる",
  "たかつ",
  "たくなう",
  "ちゃつ",
  "てう",
  "てつ",
  "っつ",
  "ってる",
  "です",
  "ないる",
  "なかつ",
  "なさう",
  "さいる",
  "ましる",
  "っる",
  "べう",
  "べす",
  "べつ",
  "やつ",
  "らる",
  "んでしる",
  "にる"
];

module.exports = {
  Form,
  ADJECTIVE_INFLECTIONS,
  ICHIDAN_INFLECTIONS,
  GODAN_INFLECTIONS,
  SURU_INFLECTIONS,
  KURU_INFLECTIONS,
  SPECIAL_INFLECTIONS,
  IKU_INFLECTIONS,
  ICHIDAN,
  BOGUS_INFLECTIONS
};
